// @ts-check

const gulp = require("gulp");
const { assets } = require("./assets");
const check = require("./check");
const { clean, clear, wipe } = require("./clean");
const configure = require("./configure");
const env = require("./env");
const i18n = require("./i18n");
const core = require("./core");
const generateRoute = require("./generate-route");
const generateComponent = require("./generate-component");
const htaccess = require("./htaccess");
const gitignore = require("./gitignore");
const pkg = require("./pkg");
const readme = require("./readme");
const setServerPort = require("./setServerPort");
const svgicons = require("./svgicons");
const watch = require("./watch");
const { ciPre, ciPost } = require("./ci");
const { tplBuild, tplWatch } = require("./tpl");

// FIXME: exit build on failure
// process.exitCode = 1;
// gulp.doneCallback = function(err) {
//   process.exit(err ? 1 : 0);
// };

// simply exposed public tasks
gulp.task(clean);
gulp.task(clear);
gulp.task(wipe);
gulp.task(core);
gulp.task(generateRoute);
gulp.task(generateComponent);
gulp.task(svgicons);
gulp.task(i18n);

// composed public tasks
gulp.task("init", function (callback) {
  if (require("ci-info").isCI) {
    callback();
  } else {
    return gulp.series(
      clean,
      env,
      htaccess,
      gulp.parallel(readme, pkg, gitignore, configure, tplBuild),
      i18n
    )(callback);
  }
});

gulp.task(
  "start",
  gulp.series(
    clean,
    check,
    setServerPort,
    env,
    htaccess,
    gulp.parallel(readme, pkg, gitignore, configure, tplBuild),
    i18n,
    svgicons,
    gulp.parallel(tplWatch, assets, watch)
  )
);

gulp.task(
  "build",
  gulp.series(
    clean,
    check,
    setServerPort,
    env,
    gulp.parallel(pkg, gitignore, tplBuild),
    i18n,
    assets,
    svgicons,
    htaccess
  )
);

gulp.task("deploy", gulp.series(ciPre, "build", ciPost));

gulp.task(
  "test",
  gulp.series("build", (callback) => {
    const log = require("@acanto/workflow-utils/log");
    const chalk = require("@acanto/workflow-utils/chalk");
    log(
      chalk.greenBright(`Production code is built, you can view it at ${process.env.APP_URL}.
        You might first temporarily set in your ".env" file APP_ENV=production.
      `)
    );
    callback();
  })
);

module.exports = gulp;
