// @ts-check
const path = require("path");
const paths = require("./paths");
const { isProduction, hasSourceMaps } = require("./config");

let webpackConfig = {
  mode: isProduction ? "production" : "development",
  bail: isProduction ? true : false,
  // devServer: require("./webpack/devServer"),
  // experiments: {
  //   asset: true
  // },
  // @see https://webpack.js.org/configuration/devtool/
  watchOptions: {
    followSymlinks: true,
  },
  devtool: hasSourceMaps() ? (isProduction ? "source-map" : "eval") : false,
  entry: require("./webpack/entry")(),
  output: require("./webpack/output"),
  optimization: require("./webpack/optimization"),
  resolve: require("./webpack/resolve"),
  // persistent cache during CI builds on filesystem
  // cache: isProduction ? {
  cache: require("ci-info").isCI
    ? {
        type: "filesystem",
        cacheDirectory: path.join(paths.root, "/.npm/.cache-webpack"),
      }
    : true,
  snapshot: {
    // add path for caching on CI
    managedPaths: [
      path.join(paths.root, "/node_modules"),
      path.join(paths.root, "/.npm"),
    ],
  },
  // for webpack-dev-server
  infrastructureLogging: {
    level: "info",
  },
  stats: {
    assets: false,
    hash: false,
    timings: false,
    chunkModules: false,
    modules: false,
    cached: false,
    cachedAssets: false,
    version: false,
    builtAt: false,
    entrypoints: false,
    excludeAssets: /.*LICENSE\.txt$/, // Exclude license files
  },
  performance: {
    // do not show warnings for too big favicons, they are autogenerated...
    // only show performance report for css/js files
    assetFilter: (assetFilename) => /\.js$|css$/.test(assetFilename),
  },
  module: {
    rules: [
      require("./webpack/rules-scripts")(),
      require("./webpack/rules-styles")(),
      require("./webpack/rules-fonts")(),
      require("./webpack/rules-images")(),
      require("./webpack/rules-media")(),
      // {
      //   test: /\.(woff2?|ttf|otf|eot|svg)$/,
      //   type: "asset",
      //   generator: {
      //     filename: "fonts/[hash][ext]"
      //   }
      // },
      // {
      //   test: /\.(gif|png|jpe?g|svg)$/i,
      //   type: "asset",
      //   generator: {
      //     filename: "images/[hash][ext]"
      //   }
      // },
      // {
      //   test: /\.(mp4|mp3|ogg|wav)$/i,
      //   type: "asset",
      //   generator: {
      //     filename: "media/[ext]"
      //   }
      // }
    ],
  },
  plugins: require("./webpack/plugins")(),
};

if (process.env.DEV_ANALYZE === "true") {
  const SpeedMeasurePlugin = require("speed-measure-webpack-plugin");
  const smp = new SpeedMeasurePlugin();
  webpackConfig = smp.wrap(webpackConfig);
}

module.exports = webpackConfig;
